% Compute Data
% by Jakkrapun Chuanasa and Wasin Rujikietgumjorn
% 02/24/05
% Last Updated : 07/06/2010

%========================================
%This is for the formulas
% Ph1(i,j) = P(i,j) - P(i,i)
% and
% Ph2(i,j) = P(j,i) - P(i,i)
%
%=========================================

format compact

try

    cd(H);                 % change directory to H where H is the beginning directory that we start program with
    try
        %     if length(B) == 0      % This means that no sheet name is specified. So, we pick the data from the first sheet
        %         M = xlsread(A);    % Use xlsread formular <----- with no sheet specified
        %     elseif length(B) ~= 0  % This means that the sheet name has been specified to value B
        %         M = xlsread(A,B);  % Use xlsread formular <----- with sheet specified
        %     end

        load(A);

        if exist('inputdata','var')
            M = inputdata;
        else
            errordlg('Error : Input variable must be named ''inputdata''','Error') ;
            T = sprintf('\n Error : Input variable must be named ''inputdata''') ;
            set(handles.info, 'String', T);
            break;
        end

        if ispc
            index1 = strfind(A,'\');
        else
            index1 = strfind(A,'/');
        end
        index2 = strfind(A,'.');
        from = index1(end)+1;
        to = index2(end)-1;
        filename = A(from:to);
    catch
        errordlg('Error : Please specify your data file','Error') ;
        T = {sprintf('\n Error : Please specify your data file')} ;
        set(handles.info, 'String', T);
        break ;
    end
    isnanerror = 0 ;
    for i = 1:size(M,1)
        for j = 1:size(M,2)
            if isnan(M(i,j))
                isnanerror = 1 ;
                break ;
            end
        end
    end
    if isnanerror
        errordlg('Error : Element in matrix is not a number','Error') ;
        T = {sprintf('\n Error : Element in matrix is not a number')} ;
        set(handles.info, 'String', T);
        break ;
    end
    M ;         % Matrix M is the squared-matrix we are computing
    Matrix = M ;        % store Matrix as original one

    %% check for labels if user uses his/her own labels
    automaticlabel = 1;
    if ~strcmp(rowlabel,'empty');
        index = strfind(rowlabel,';');
        if length(index) ~= size(M,1)
           errordlg('Number of labels does not match with a dimension of input data','Error'); 
           T = {sprintf('\n Error : Number of labels does not match with a dimension of input data')} ;
           set(handles.info, 'String', T);
           break;
        end
        index = strfind(rowlabel,';');
        index2 = strfind(collabel,';');
        rowlabeleach = cell(length(index),1);
        collabeleach = cell(length(index2),1);
        cnt1 = 1;
        cnt2 = 1;
        for i = 1:length(index)
            rowlabeleach{i} = rowlabel(cnt1:index(i)-1);
            collabeleach{i} = collabel(cnt2:index2(i)-1);
            cnt1 = index(i)+1;
            cnt2 = index2(i)+1;
        end
        automaticlabel = 0;     % use his/her own labels
    end
    
    %%% checking regularity
    maximal = str2num(maximal);
    minimal = str2num(minimal);
    [Mshuffle,data,violate,T] = checkRegular(M, maximal, minimal) ;
    if forcepse == 0    % no forcing PSEs
        if minimal == 1 | maximal == 1    % if user choose to check for regular minimal or maximal
            M = Mshuffle ;
            if violate == 1
                if minimal
                    set(handles.info, 'String',T);    % Show text T (information)
                    errordlg(T,'Warning') ;
                    break ;
                else
                    set(handles.info, 'String',T);    % Show text T (information)
                    errordlg(T,'Warning') ;
                    break ;
                end
            end
        end
    else  % force PSEs
        for i = 1:size(M,1)
            data(i,1) = i ;      
            if forcepse == 1    % force it canonical
                data(i,2) = i ;   
            else                % force it with input PSEs
                for k = 1:size(M,1)
                    index = find(strcmp(pseeach,collabeleach{k}) == 1);
                    data(k,2) = index;
                end
            end
            if mod(i,26) == 0
                data(i,3) = 90 ;        % letter in ascii, A,B,C,...
                data(i,4) = ceil(i/26) ;    % number 1,2... then col 3 and 4 can combine to A1, B1 ...
            else
                data(i,3) = mod(i,26)+64 ;
                data(i,4) = ceil(i/26) ;
            end
        end
    end
    
    if tran == 1            % if user first transform the data to PSI format value bet. 0 and 1
        warning off;
        [M, warn1_1, warn2_1, warn3_1, warn4_1, T] = transform(M,strrep(eqtext,';','')) ;
        warning on;
    end
    
    if warn4_1                % warning factorial value is too large
        set(handles.info, 'String',T);    % Show text T (information)
        errordlg(sprintf('Error - 1st Transformation :\n\nFactorial value is too large'),'Error') ;
        break ;
    end
    if warn3_1                % warning input incorrect syntax
        set(handles.info, 'String',T);    % Show text T (information)
        errordlg(sprintf('Error - 1st Transformation :\n\nIncorrect Syntax'),'Error') ;
        break ;
    end
    if warn1_1              % warning not real number
        set(handles.info, 'String',T);    % Show text T (information)
        errordlg(sprintf('Error - 1st Transformation :\n\nNot a real number'),'Error') ;
        break ;
    end
    if warn2_1                % warning infinite value
        set(handles.info, 'String',T);    % Show text T (information)
        errordlg(sprintf('Error - 1st Transformation :\n\nInfinite value'),'Error') ;
        break ;
    end
    
    
    Mpsi = M ;
    M_nochange = M;

%     if length(find(M > 1 | M < 0)) > 0   % check for PSI format
%         errordlg('Error : Your data should be transformed into PSI(A,B) format','Error') ;
%         T = {sprintf('\n Error : Your data should be transformed into PSI(A,B) format')} ;
%         set(handles.info, 'String', T);
%         break ;
%     end

    if changeData == 1      % if user change value 0 and/or value 1
        if flag_setfloor == 1 || flag_setceiling == 1
            M = change01(M,change0,change1) ;
            M_nochange = M;
        end
        if flag_setpsilimit == 1
            M_nochange = M;
            index = find(diag(M) > (psi_upperlimit));
            if ~isempty(index)
                errordlg('Error : Upper limit for PSI is too low','Error') ;
                T = {sprintf('\n Error : Upper limit for PSI is too low')} ;
                set(handles.info, 'String', T);
                break ;
            end
            M = change01_2(M,psi_upperlimit) ; 
        end
    end

    if tran2 == 1            % if user second transform the data
        [M, warn1_2, warn2_2, warn3_2, warn4_2, T] = transform(M,strrep(eqtext2,';','')) ;
        [M_nochange, warn1_22, warn2_22, warn3_22, warn4_22, Tt] = transform(M_nochange,strrep(eqtext2,';','')) ;
    end
    
    if warn4_2                % warning factorial value is too large
        set(handles.info, 'String',T);    % Show text T (information)
        errordlg(sprintf('Error - 2nd Transformation :\n\nFactorial value is too large'),'Error') ;
        break ;
    end
    if warn3_2                % warning input incorrect syntax
        set(handles.info, 'String',T);    % Show text T (information)
        errordlg(sprintf('Error - 2nd Transformation :\n\nIncorrect Syntax'),'Error') ;
        break ;
    end
    if warn1_2              % warning not real number
        set(handles.info, 'String',T);    % Show text T (information)
        errordlg(sprintf('Error - 2nd Transformation :\n\nNot a real number'),'Error') ;
        break ;
    end
    if warn2_2                % warning infinite value
        set(handles.info, 'String',T);    % Show text T (information)
        errordlg(sprintf('Error - 2nd Transformation :\n\nInfinite value'),'Error') ;
        break ;
    end

    %---------------------------------
    %Define some matrices
    %--------------------------

    d_length=length(M);                            % d_length is the size of the square-matrix
    FromAtoB_1 = zeros(d_length);                  % Assign values to all the data to be all zeros
    FromAtoB_2 = zeros(d_length);
    Path_AB_1 = zeros(d_length);
    Path_AB_2 = zeros(d_length);

    for i=1:d_length;       % Run the formula 1 and 2
        for j=1:d_length;
%             P(i,j) = abs(M(i,j) - M(i,i));     % This is Ph1 formula
%             W(i,j) = abs(M(j,i) - M(i,i));     % This is Ph2 formula
            P(i,j) = max(M(i,j) - M(i,i),0);     % This is Ph1 formula
            W(i,j) = max(M(j,i) - M(i,i),0);     % This is Ph2 formula
        end
    end
    
    index = find(P ~= 0);
    if isempty(index)
        if minimal == 1
            errordlg('The matrix of increments is zero','Warning');
            T = {sprintf('\n Warning : The matrix of increments is zero')} ;
        else
            errordlg('The matrix of increments is zero','Warning');
            T = {sprintf('\n Warning : The matrix of increments is zero')} ;
        end
        set(handles.info, 'String', T);
        break;
    end

    if flag_inc == 1    % set upper limit for increment matrix
        index_incP = find(P > str2num(inc_upperlimit));
        P(index_incP) = Inf*ones(size(index_incP));

        index_incW = find(W > str2num(inc_upperlimit));
        W(index_incW) = Inf*ones(size(index_incW));
    end

    gui_active(1);      % This will add an abort button  for a progress bar (Tell us the progress of the computation)
    h_pbar = progressbar( [],0,'Progress','Computing' );   % 0 means begins from zero. Show word 'Progress' on the bar. Title bar is named 'Computing'.
    max_count = d_length+d_length;     % 100% of bar process amount
    if flagmds == 1
        max_count = max_count+1;
    end
    if flagkm == 1
        max_count = max_count+1;
    end
    if flaghc == 1
        max_count = max_count+1;
    end

    % set position
    FigPos=get(0,'DefaultFigurePosition');
    OldUnits = get(h_pbar, 'Units');
    set(h_pbar, 'Units', 'pixels');
    OldPos = get(h_pbar,'Position');
    FigWidth = OldPos(3);
    FigHeight = OldPos(4);
    if isempty(gcbf)
        ScreenUnits=get(0,'Units');
        set(0,'Units','pixels');
        ScreenSize=get(0,'ScreenSize');
        set(0,'Units',ScreenUnits);

        FigPos(1)=1/2*(ScreenSize(3)-FigWidth);
        FigPos(2)=2/3*(ScreenSize(4)-FigHeight);
    else
        GCBFOldUnits = get(gcbf,'Units');
        set(gcbf,'Units','pixels');
        GCBFPos = get(gcbf,'Position');
        set(gcbf,'Units',GCBFOldUnits);
        FigPos(1:2) = [(GCBFPos(1) + GCBFPos(3) / 2) - FigWidth / 2, ...
            (GCBFPos(2) + GCBFPos(4) / 2) - FigHeight / 2];
    end
    FigPos(3:4)=[FigWidth FigHeight];
    set(h_pbar, 'Position', FigPos);
    set(h_pbar, 'Units', OldUnits);

    waitcount = 0 ;                 % value for update waitbar counting
    start_time = datestr(now);           %this gives date and time when start
    t0 = clock;       % gives the start time ...CLOCK = [year month day hour minute seconds]

    % construct node ID
    for i = 1:d_length
        nodeletter = mod(i,26);
        if nodeletter == 0
            nodeletter = 'Z';
        else
            nodeletter = char(mod(i,26)+64);
        end
        nodenumber = ceil(i/26.001);
        nodeid{i} = [nodeletter num2str(nodenumber)];
    end

    for x=1:d_length;          % Start computing. Begin with loop X = 1 to the size of the matrix (Row)

        [dist,path] = findshortestpath(P,x,nodeid);
        FromAtoB_1(x,:) = dist;
        Path_AB_1_text(x,:) = path;

        if compute_type == 0         %that means we are using long method just go ahead and compute more
            [dist,path] = findshortestpath(W,x,nodeid);
            FromAtoB_2(x,:) = dist;
            Path_AB_2_text(x,:) = path;
        end        % End Compute short or long

        if gui_active
            h_pbar = progressbar( h_pbar,1/max_count); % update progressbar
        else
            break ;
        end
    end    % End X

    if ~gui_active                % this is for the progress bar
        T = {sprintf('\nOperation was aborted by user')} ;
        set(handles.info, 'String',T);    % Show text T (information)
        progressbar( h_pbar,-1 );          % finish progress bar
        break ;
    end

    if flag_inc == 1
        check = 1;
        for i = 1:size(Path_AB_1_text,1)
            for j = 1:size(Path_AB_1_text,2)
                if length(Path_AB_1_text{i,j}) == 0
                    check = 0;
                    row = nodeid{i};
                    col = nodeid{j};
                    break;
                end
            end
        end
        if check == 0
            close(h_pbar);
            Err = {sprintf('Error : There is no path connecting %s to %s.',row,col);
                sprintf('Upper limit for PSI or upper limit for ZETA-increments is too low.')};
            errordlg(Err,'Error') ;
            Err = sprintf('\n %s',Err{:});
            set(handles.info, 'String',Err);    % Show text T (information)
            break;
        end
    end

    %%%%%%%%%%%%%%%%%%%% compute Loop from Path  %%%%%%%%%%%%%%%%%%
    for i = 1:d_length
        for j = 1:d_length
            if i == j
                Loop_AB_1{i,j} = Path_AB_1_text{i,j} ;
            else
                begin = Path_AB_1_text{i,j} ;
                begin = begin(1:2) ;
                Loop_AB_1{i,j} = [Path_AB_1_text{i,j} Path_AB_1_text{j,i} begin] ;
            end

            if compute_type == 0
                if i == j
                    Loop_AB_2{i,j} = Path_AB_2_text{i,j} ;
                else
                    begin = Path_AB_2_text{i,j} ;
                    begin = begin(1:2) ;
                    Loop_AB_2{i,j} = [Path_AB_2_text{i,j} Path_AB_2_text{j,i} begin] ;
                end
            end
            % compute S(A,B)
            %         s(i,j) = abs(M(i,j) + M(j,i)-M(i,i)-M(j,j));   %I tried to put abs before we compute (Don't want negative results)
%             s(i,j) = abs(M_nochange(i,j) + M_nochange(j,i)-M_nochange(i,i)-M_nochange(j,j));   % avoid Undefined values, so use original S
            s(i,j) = (M_nochange(i,j) + M_nochange(j,i)-M_nochange(i,i)-M_nochange(j,j));
        end  % end j
        if gui_active
            h_pbar = progressbar( h_pbar,1/max_count); % update progressbar
        else
            break ;
        end
    end   % end i
    S = s' ;
    index = find(S < 0);
    S(index) = zeros(length(index),1);

    if ~gui_active                % this is for the progress bar
        T = {sprintf('\nOperation was aborted by user')} ;
        set(handles.info, 'String',T);    % Show text T (information)
        progressbar( h_pbar,-1 );          % finish progress bar
        break ;
    end
    
    % compute G(1)(A,B)
    FromAtoB_1_transpose = FromAtoB_1';
    G = FromAtoB_1+FromAtoB_1_transpose;
    
    % compute G(2)(A,B)
    longcom_g1g2 = 0;
    if compute_type == 0    % if we compute long
        FromAtoB_2_transpose = FromAtoB_2';
        GFech2 = FromAtoB_2+FromAtoB_2_transpose;
        GCheck = G-GFech2;
        index = find(abs(GCheck) > 1e-6);
        if ~isempty(index)
            [push,optionselect] = warningCheck;
            if strcmp(push,'Ok')
                if optionselect == 1   % max
                    G = reshape(max([reshape(G,1,numel(G)); reshape(GFech2,1,numel(GFech2))]),size(G,1),size(G,2));
                    GFech2 = G;
                elseif optionselect == 2   % mean
                    G = reshape(mean([reshape(G,1,numel(G)); reshape(GFech2,1,numel(GFech2))]),size(G,1),size(G,2));
                    GFech2 = G;
                else
                    longcom_g1g2 = 1;
                end
            else
                progressbar( h_pbar,-1 );
                T = {sprintf('\nOperation was aborted by user')} ;
                set(handles.info, 'String',T);    % Show text T (information)
                break;
            end
        end
    end

    %%%%%%%%%%%% Post Analysis %%%%%%%%%%%%%

    chkErrormds = 0;
    chkErrormds2 = 0;
    chkErrorkm = 0;
    chkErrorkm2 = 0;
    chkErrorhc = 0;
    chkErrorhc2 = 0;

    ptletter = {'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S',...
        'T','U','V','W','X','Y','Z'};
    for i = 1:size(G,1)
        cnt1 = mod(i,26);
        if cnt1 == 0
            cnt1 = 26;
        end
        cnt2 = ceil(i/26.001);
        ptid{i,1} = [char(ptletter(cnt1)) num2str(cnt2)];
    end

    Sori = S;

    %%% create result folder
    chk = 0;
    cnt = 0;
    while chk == 0
        if cnt ~= 0
            foldername = [datapath filename '(' num2str(cnt) ')'];
        else
            foldername = [datapath filename];
        end
        if exist(foldername,'dir')
            cnt = cnt+1;
        else
            mkdir(foldername);
            chk = 1;
        end
    end

    %%% Post Analysis 
    if longcom_g1g2 == 1    % option : Both
        postanalysis2;
    else  % options : MAX, MEAN or normal
        postanalysis1;
    end

    %%% Close figure
    if flagmds == 1 | flagkm == 1 | flaghc == 1
        S = Sori;
    end

    stop_time = datestr(now);

    runtime = etime(clock,t0);  %Time that elapse for computation ...t0 = start time. CLOCK = [year month day hour minute seconds]
    T = {sprintf('\n-Start time = %s', start_time);...         % T is text message
        sprintf('-Stop time = %s', stop_time);...
        sprintf('-Total time = %g seconds', runtime);...
        sprintf('-----------------------------------------------------------');};
    if minimal == 1 || maximal == 1
        if minimal == 1
            Ttemp = {'Checked for regular minimality'} ;
        else
            Ttemp = {'Checked for regular maximality'} ;
        end
        T = [T; Ttemp] ;
    end
    if tran == 1
        eqtext = ['Psi = ' eqtext] ;
        Ttemp = {sprintf('1st Transformation : %s', strrep(eqtext,';',''))} ;
        T = [T; Ttemp] ;
        T2 = Ttemp;
    else
        Ttemp = {sprintf('1st Transformation : None')} ;
        T = [T; Ttemp] ;
        T2 = Ttemp;
    end
    if tran2 == 1
        eqtext2 = ['Y = ' eqtext2] ;
        eqtext2 = strrep(eqtext2,'Y','Zeta') ;
        eqtext2 = strrep(eqtext2,'X','Psi') ;
        Ttemp = {sprintf('2nd Transformation : %s', strrep(eqtext2,';',''))} ;
        T = [T; Ttemp] ;
%         T2 = [Ttemp] ;
        T2 = [T2; Ttemp] ;
    else
        Ttemp = {sprintf('2nd Transformation : None')} ;
        T = [T; Ttemp] ;
%         T2 = [Ttemp] ;
        T2 = [T2; Ttemp] ;
    end
    temp = [];
    if changeData == 1
        if flag_setfloor == 1
            Ttemp = {sprintf('PSI floor = %0.3f',change0)} ;
            temp = strcat(temp,Ttemp);
        end
        if flag_setceiling == 1
            if flag_setfloor == 1
                temp = strcat(temp,{', '});
            end
            Ttemp = {sprintf('PSI ceiling = %0.3f',change1)};
            temp = strcat(temp,Ttemp);
        end
        if flag_setpsilimit == 1
            if flag_setfloor == 1 || flag_setceiling == 1
                temp = strcat(temp,{', '});
            end
            Ttemp = {sprintf('PSI limit = %0.3f',psi_upperlimit)};
            temp = strcat(temp,Ttemp);
        end
        if flag_inc == 1
            if flag_setfloor == 1 || flag_setceiling == 1 || flag_setpsilimit == 1
                temp = strcat(temp,{', '});
            end
            Ttemp = {sprintf('INC limit = %0.3f',str2num(inc_upperlimit))} ;
            temp = strcat(temp,Ttemp);
        end
        T = [T; temp] ;
        T2 = [T2; temp] ;
    else
        Ttemp = [];
%         temp = {sprintf('0 and 1 Unchanged')} ;
        if flag_inc == 1
            Ttemp = {sprintf(', INC limit = %0.3f',str2num(inc_upperlimit))} ;
        end
        temp = strcat(temp,Ttemp);
        T = [T; temp] ;
        T2 = [T2; temp] ;
    end
    if flagmds == 1 | flagkm == 1 | flaghc == 1
        if flagmds == 1
            text1 = 'MDS';
        else
            text1 = '';
        end
        if flagkm == 1
            text2 = 'K-Means';
        else
            text2 = '';
        end
        if flaghc == 1
            text3 = 'Hierarchical cluster analysis';
        else
            text3 = '';
        end
        Ttemp = {sprintf('Post Analysis : %s %s %s', text1,text2,text3)} ;
        T = [T; Ttemp] ;
    else
        Ttemp = {sprintf('Post Analysis : No selection')} ;
        T = [T; Ttemp] ;
    end
    if chkErrormds == 0 & chkErrormds2 == 0 & chkErrorkm == 0 & chkErrorkm2 == 0 & chkErrorhc == 0 & chkErrorhc2 == 0
        Ttemp = {sprintf('\n'); sprintf('The results were computed successfully.')};
    else
        if chkErrormds == 1 | chkErrormds2 == 1
            Ttemp = {sprintf('Error in MDS : See MATLAB windows for more details')};
        end
        if chkErrorkm == 1 | chkErrorkm2 == 1
            Ttemp = {sprintf('Error in KMEANS : See MATLAB windows for more details')};
        end
        if chkErrorhc == 1 | chkErrorhc2 == 1
            Ttemp = {sprintf('Error in HCA : See MATLAB windows for more details')};
        end
        Ttemp = [Ttemp; ' '; sprintf('The results were computed successfully.')];
    end
    T = [T; Ttemp] ;

    %---------------------------------------------------------------------
    %%% description for post analysis
    %---------------------------------------------------------------------
    T3 = {''};
    if flagmds == 1
        T3 = {sprintf('*** MDS ***')};
        T3 = [T3; {sprintf('Method (G,S) : %s,%s',opt_method,opt_method2)}];
        T3 = [T3; {sprintf('Dimension (G,S) : %d to %d, %d to %d',dimmdsfrom,dimmdsto,dimmds2from,dimmds2to)}];
        T3 = [T3; {sprintf('Start (G,S) : %s,%s',opt_start,opt_start2)}];
        T3 = [T3; {sprintf('Replicates (G,S) : %d,%d',repmds,repmds2)}];
        T3 = [T3; {sprintf('Display (G,S) : %s,%s',opt_display,opt_display2)}];
        T3 = [T3; {sprintf('Max Iterations (G,S) : %d,%d',maxitermds,maxitermds2)}];
        T3 = [T3; {sprintf('Tolerance Function (G,S) : %d,%d',tolfunmds,tolfunmds2)}];
        T3 = [T3; {sprintf('Tolerance Parameters (G,S) : %d,%d',tolxmds,tolxmds2)}];
        T3 = [T3; {sprintf(' ')}];
    end
    if flagkm == 1 && flagmds == 1
        T3 = [T3; {sprintf('*** KMEANS ***')}];
        T3 = [T3; {sprintf('Cluster : %d',cluskm)}];
        T3 = [T3; {sprintf('Empty Action : %s',opt_empty)}];
        T3 = [T3; {sprintf('Replicates : %d',repkm)}];
        T3 = [T3; {sprintf('Start : %s',opt_startkm)}];
        T3 = [T3; {sprintf('Display : %s',opt_displaykm)}];
        T3 = [T3; {sprintf('Max Iterations : %d',maxiterkm)}];
        T3 = [T3; {sprintf(' ')}];
    end
    if flaghc == 1
        T3 = [T3; {sprintf('*** Hierarchical Clustering ***')}];
        T3 = [T3; {sprintf('Linkage : %s',opt_linkage)}];
        T3 = [T3; {sprintf('Criterion : %s',opt_criterion)}];
        if methodthresholdhc == 1
            T3 = [T3; {sprintf('Max Clusters : %d',thresholdhc)}];
        elseif methodthresholdhc == 2
            T3 = [T3; {sprintf('Cutoff : %f',thresholdhc)}];
        end
        T3 = [T3; {sprintf('Depth : %d',depthhc)}];
        T3 = [T3; {sprintf('\n Remark.1 : The cophenetic correlation for a cluster tree is defined as the linear correlation')}];
        T3 = [T3; {sprintf('coefficient between the cophenetic distances obtained from the tree, and the original distances (or')}];
        T3 = [T3; {sprintf('dissimilarities) used to construct the tree. Thus, it is a measure of how faithfully the tree')}];
        T3 = [T3; {sprintf('represents the dissimilarities among observations.')}];
        T3 = [T3; {sprintf('Remark.2 : Inconsistent Function, Col.1:Mean of the heights of all the links included in the')}];
        T3 = [T3; {sprintf('calculation, Col.2:standard deviation of all the links included in the calculation, Col.3:number of')}];
        T3 = [T3; {sprintf('links included in the calculation, Col.4:inconsistent coefficient.')}];
    end

    %% prepare data and plots
    % PSE
    PSE = cell(size(data,1)+1,3);
    PSE(1,:) = {'Observation area 1','Observation area 2','Common Label'};
    for i = 1:size(data,1)
        if automaticlabel == 1
            temp = data(i,1);
            t1 = mod(temp,26);
            t2 = ceil(temp/26);
            if t1 ~= 0
                t1 = char(t1+96);
            else
                t1 = 'z';
            end
            PSE{i+1,1} = [t1 num2str(t2)];

            temp = data(i,2);
            t1 = mod(temp,26);
            t2 = ceil(temp/26);
            if t1 ~= 0
                t1 = char(t1+96);
            else
                t1 = 'z';
            end
            PSE{i+1,2} = [t1 num2str(t2)];
        else
            PSE(2:end,1) = rowlabeleach(data(:,1));
            PSE(2:end,2) = collabeleach(data(:,2));
        end

        t1 = char(data(i,3));
        t2 = num2str(data(i,4));
        PSE{i+1,3} = [t1 t2];
    end

    % PSI matrix
    PSI = cell(size(Mpsi)+1);
    PSI(2:end,2:end) = num2cell(Mpsi);
    PSI(2:end,1) = ptid;
    PSI(1,2:end) = ptid;

    % ZETA matrix
    ZETA = cell(size(M)+1);
%     ZETA(2:end,2:end) = num2cell(M);
    ZETA(2:end,2:end) = num2cell(M_nochange);   % not show Inf value when set limit PSI
    ZETA(2:end,1) = ptid;
    ZETA(1,2:end) = ptid;

    % G matrix
    G_matrix = {};
    if compute_type == 1
        G_matrix = cell(size(G)+1);
        G_matrix(2:end,2:end) = num2cell(G);
        G_matrix(2:end,1) = ptid;
        G_matrix(1,2:end) = ptid;
    end

    % Loop
    Loop = {};
    if compute_type == 1
        Loop = cell(size(Loop_AB_1)+1);
        Loop(2:end,2:end) = Loop_AB_1;
        Loop(2:end,1) = ptid;
        Loop(1,2:end) = ptid;
    end

    % S matrix
    S_matrix = cell(size(S)+1);
    S_matrix(2:end,2:end) = num2cell(S);
    S_matrix(2:end,1) = ptid;
    S_matrix(1,2:end) = ptid;

    % S vs G plot
    St = squareform(S);
    Gt = squareform(G);
    figure(cntfigall); clf(cntfigall); hold on;
    plot(St,Gt,'bo','markersize',5,'linewidth',1);
    max_plot = max([St Gt]);
    plot([0 max_plot],[0 max_plot],'r-');
    titletext = ['The Chart for ' num2str(size(G,1)) 'x' num2str(size(G,1)) ' Matrix Level 1'];
    title(titletext);
    if longcom_g1g2 == 0
        axis equal; xlabel('S-index'); ylabel('Fechnerian Distance (G)');
        axis([0 max_plot 0 max_plot]);
        legend('S vs G','Diagonal Line',2);
        FIG_out = [foldername '/Figure_S_vs_G'];
    else
        axis equal; xlabel('S-index'); ylabel('Fechnerian Distance 1 (G1)');
        axis([0 max_plot 0 max_plot]);
        legend('S vs G1','Diagonal Line',2);
        FIG_out = [foldername '/Figure_S_vs_G1'];
    end
    saveas(gcf,FIG_out);
%     print('-f','-djpeg','-r300',FIG_out);
    cntfigall = cntfigall+1;
    
    if longcom_g1g2 == 1
        St = squareform(S);
        GtFech2 = squareform(GFech2);
        figure(cntfigall); clf(cntfigall); hold on;
        plot(St,GtFech2,'bo','markersize',5,'linewidth',1);
        max_plot = max([St GtFech2]);
        plot([0 max_plot],[0 max_plot],'r-');
        axis equal; xlabel('S-index'); ylabel('Fechnerian Distance 2 (G2)');
        axis([0 max_plot 0 max_plot]);
        legend('S vs G2','Diagonal Line',2);
        titletext = ['The Chart for ' num2str(size(GFech2,1)) 'x' num2str(size(GFech2,1)) ' Matrix Level 1'];
        title(titletext);
        FIG_out = [foldername '/Figure_S_vs_G2'];
        saveas(gcf,FIG_out);
        %     print('-f','-djpeg','-r300',FIG_out);
        cntfigall = cntfigall+1;
    end

    if longcom_g1g2 == 0
        createtable;
    else
        createtable2;
    end

    % long computation
    G_1 = {};
    G_2 = {};
    G1 = {};
    G2 = {};
    Check = {};
    Path_1 = {};
    Path_2 = {};
    Loop_1 = {};
    Loop_2 = {};

    if compute_type == 0
        G_1 = cell(size(G)+1);
        G_1(2:end,2:end) = num2cell(FromAtoB_1);
        G_1(1,2:end) = ptid;
        G_1(2:end,1) = ptid;

        G_2 = cell(size(G)+1);
        G_2(2:end,2:end) = num2cell(FromAtoB_2);
        G_2(1,2:end) = ptid;
        G_2(2:end,1) = ptid;

        G1 = cell(size(G)+1);
%         G1(2:end,2:end) = num2cell(FromAtoB_1+FromAtoB_1');
        G1(2:end,2:end) = num2cell(G);
        G1(1,2:end) = ptid;
        G1(2:end,1) = ptid;

        G2 = cell(size(G)+1);
%         G2(2:end,2:end) = num2cell(FromAtoB_2+FromAtoB_2');
        G2(2:end,2:end) = num2cell(GFech2);
        G2(1,2:end) = ptid;
        G2(2:end,1) = ptid;

        Check = cell(size(G)+1);
%         diff = (FromAtoB_1+FromAtoB_1')-(FromAtoB_2+FromAtoB_2');
        diff = G-GFech2;
        Check(2:end,2:end) = num2cell(diff);
        Check(1,2:end) = ptid;
        Check(2:end,1) = ptid;

        Path_1 = cell(size(G)+1);
        Path_1(2:end,2:end) = Path_AB_1_text;
        Path_1(1,2:end) = ptid;
        Path_1(2:end,1) = ptid;

        Path_2 = cell(size(G)+1);
        Path_2(2:end,2:end) = Path_AB_2_text;
        Path_2(1,2:end) = ptid;
        Path_2(2:end,1) = ptid;

        Loop_1 = cell(size(G)+1);
        Loop_1(2:end,2:end) = Loop_AB_1;
        Loop_1(1,2:end) = ptid;
        Loop_1(2:end,1) = ptid;

        Loop_2 = cell(size(G)+1);
        Loop_2(2:end,2:end) = Loop_AB_2;
        Loop_2(1,2:end) = ptid;
        Loop_2(2:end,1) = ptid;
    end

    %% assign variables to workspace, find for this command for HOURS!!!
    assignin('base','PSE',PSE);
    assignin('base','PSI',PSI);
    assignin('base','ZETA',ZETA);

    assignin('base','G_matrix',G_matrix);
    assignin('base','Loop',Loop);

    assignin('base','S_matrix',S_matrix);
    assignin('base','S_vs_G_Table',S_vs_G_Table);

    assignin('base','MDS_Table',MDS_Table);
    assignin('base','KMEANS_Table',KMEANS_Table);
    assignin('base','HCA_Table',HCA_Table);

    assignin('base','G_1',G_1);
    assignin('base','G_2',G_2);
    assignin('base','G1',G1);
    assignin('base','G2',G2);
    assignin('base','Check',Check);
    assignin('base','Path_1',Path_1);
    assignin('base','Path_2',Path_2);
    assignin('base','Loop_1',Loop_1);
    assignin('base','Loop_2',Loop_2);

    %% save results
    if flagmds == 0 & flagkm == 0 & flaghc == 0
        if compute_type == 1
            save([foldername '/Tables'],'PSE','PSI','ZETA','G_matrix','Loop','S_matrix','S_vs_G_Table');
        else
            save([foldername '/Tables'],'PSE','PSI','ZETA','S_matrix','S_vs_G_Table', ...
                'G_1','G_2','G1','G2','Check','Path_1','Path_2','Loop_1','Loop_2');
        end
    elseif flagmds == 1 & flagkm == 0 & flaghc == 0
        if compute_type == 1
            save([foldername '/Tables'],'PSE','PSI','ZETA','G_matrix','Loop','S_matrix','S_vs_G_Table','MDS_Table');
        else
            save([foldername '/Tables'],'PSE','PSI','ZETA','S_matrix','S_vs_G_Table', ...
                'G_1','G_2','G1','G2','Check','Path_1','Path_2','Loop_1','Loop_2','MDS_Table');
        end
    elseif flagmds == 0 & flagkm == 1 & flaghc == 0
        if compute_type == 1
            save([foldername '/Tables'],'PSE','PSI','ZETA','G_matrix','Loop','S_matrix','S_vs_G_Table','KMEANS_Table');
        else
            save([foldername '/Tables'],'PSE','PSI','ZETA','S_matrix','S_vs_G_Table', ...
                'G_1','G_2','G1','G2','Check','Path_1','Path_2','Loop_1','Loop_2','KMEANS_Table');
        end
    elseif flagmds == 0 & flagkm == 0 & flaghc == 1
        if compute_type == 1
            save([foldername '/Tables'],'PSE','PSI','ZETA','G_matrix','Loop','S_matrix','S_vs_G_Table','HCA_Table');
        else
            save([foldername '/Tables'],'PSE','PSI','ZETA','S_matrix','S_vs_G_Table', ...
                'G_1','G_2','G1','G2','Check','Path_1','Path_2','Loop_1','Loop_2','HCA_Table');
        end
    elseif flagmds == 1 & flagkm == 1 & flaghc == 0
        if compute_type == 1
            save([foldername '/Tables'],'PSE','PSI','ZETA','G_matrix','Loop','S_matrix','S_vs_G_Table','MDS_Table','KMEANS_Table');
        else
            save([foldername '/Tables'],'PSE','PSI','ZETA','S_matrix','S_vs_G_Table', ...
                'G_1','G_2','G1','G2','Check','Path_1','Path_2','Loop_1','Loop_2','MDS_Table','KMEANS_Table');
        end
    elseif flagmds == 1 & flagkm == 0 & flaghc == 1
        if compute_type == 1
            save([foldername '/Tables'],'PSE','PSI','ZETA','G_matrix','Loop','S_matrix','S_vs_G_Table','MDS_Table','HCA_Table');
        else
            save([foldername '/Tables'],'PSE','PSI','ZETA','S_matrix','S_vs_G_Table', ...
                'G_1','G_2','G1','G2','Check','Path_1','Path_2','Loop_1','Loop_2','MDS_Table','HCA_Table');
        end
    elseif flagmds == 0 & flagkm == 1 & flaghc == 1
        if compute_type == 1
            save([foldername '/Tables'],'PSE','PSI','ZETA','G_matrix','Loop','S_matrix','S_vs_G_Table','KMEANS_Table','HCA_Table');
        else
            save([foldername '/Tables'],'PSE','PSI','ZETA','S_matrix','S_vs_G_Table', ...
                'G_1','G_2','G1','G2','Check','Path_1','Path_2','Loop_1','Loop_2','KMEANS_Table','HCA_Table');
        end
    elseif flagmds == 1 & flagkm == 1 & flaghc == 1
        if compute_type == 1
            save([foldername '/Tables'],'PSE','PSI','ZETA','G_matrix','Loop','S_matrix','S_vs_G_Table','MDS_Table','KMEANS_Table','HCA_Table');
        else
            save([foldername '/Tables'],'PSE','PSI','ZETA','S_matrix','S_vs_G_Table', ...
                'G_1','G_2','G1','G2','Check','Path_1','Path_2','Loop_1','Loop_2','MDS_Table','KMEANS_Table','HCA_Table');
        end
    end

    %% display results
    openvar('PSE');
    openvar('PSI');
    if tran2 == 1 | changeData == 1
        openvar('ZETA');
    end
    if compute_type == 1
        openvar('G_matrix');
        openvar('Loop');
    else
        openvar('G_1');
        openvar('G1');
        openvar('G_2');
        openvar('G2');
        openvar('Check');
        openvar('Path_1');
        openvar('Path_2');
        openvar('Loop_1');
        openvar('Loop_2');
    end
    openvar('S_matrix');
    openvar('S_vs_G_Table');
    if flagmds == 1
        openvar('MDS_Table');
    end
    if flagkm == 1 && flagmds == 1
        openvar('KMEANS_Table');
    end
    if flaghc == 1
        openvar('HCA_Table');
    end

    %% print summary of test
%     fid = fopen(['../' foldername '/Summary.txt'],'wt');
    fid = fopen([foldername '/Summary.txt'],'wt');
    fprintf('\n');
    fprintf(fid,'\n');
    testdate = clock;   % [year month day hour minute second]
    fprintf(' *** SUMMARY : %d/%d/%d %d:%d:%0.0f *** \n',testdate([2 3 1 4 5 6]));
    fprintf(' ========== Transformation ============================================================= \n');
    fprintf(fid,' *** SUMMARY : %d/%d/%d %d:%d:%0.0f *** \n',testdate([2 3 1 4 5 6]));
    fprintf(fid,' ========== Transformation ============================================================= \n');
    for i = 1:size(T2,1)
        fprintf(' %s\n',T2{i});
        fprintf(fid,' %s\n',T2{i});
    end
    fprintf(' ======================================================================================= \n\n');
    fprintf(' =========== Post Analysis ============================================================= \n');
    fprintf(fid,' ======================================================================================= \n\n');
    fprintf(fid,' =========== Post Analysis ============================================================= \n');
    if isempty(char(T3))
        fprintf(' None\n');
        fprintf(fid,' None\n');
    else
        for i = 1:size(T3,1)
            fprintf(' %s\n',T3{i});
            fprintf(fid,' %s\n',T3{i});
        end
    end
    fprintf(' ======================================================================================= \n');
    fprintf(' Note: use command ''close all'' to close all result figures\n');
    fprintf('       use command ''clear'' to remove all variables before rerun the program\n');
    fprintf('       use command ''clc'' to clear a command window\n\n');
    fprintf(fid,' ======================================================================================= \n');
    fprintf(fid,' Note: use command ''close all'' to close all result figures\n');
    fprintf(fid,'       use command ''clear'' to remove all variables before rerun the program\n');
    fprintf(fid,'       use command ''clc'' to clear a command window\n\n');
    fclose(fid);

catch
    err = lasterror;
    st1 = err.stack(1,1);
    T = {sprintf(' *** Error *** \n Message: %s \n Identifier: %s \n File: %s \n Line: %d', ...
        err.message,err.identifier,st1.file,st1.line)} ;
    set(handles.info, 'String', T);
    fprintf('%s\n',T{:});
    %     progressbar( h_pbar,-1 );          % finish progress bar

    %%% close all previous figure;
    all_figs = findobj(0, 'type', 'figure');
    if length(all_figs) > 1
        close(all_figs(1:end-1));
    end

    %%% remove saved folder
%     curr_path = pwd;
%     cd(curr_path(1:end-10));
%     rmdir(foldername,'s');
%     cd(curr_path);

    break;
end